=== modified file 'bzrlib/cleanup.py'
--- bzrlib/cleanup.py	2009-10-26 06:23:14 +0000
+++ bzrlib/cleanup.py	2010-01-07 01:30:20 +0000
@@ -116,6 +116,10 @@
         return _do_with_cleanups(
             self.cleanups, self.func, self, *args, **kwargs)
 
+    def run_simple(self, *args, **kwargs):
+        return _do_with_cleanups(
+            self.cleanups, self.func, *args, **kwargs)
+
 
 def _do_with_cleanups(cleanup_funcs, func, *args, **kwargs):
     """Run `func`, then call all the cleanup_funcs.

=== modified file 'bzrlib/reconcile.py'
--- bzrlib/reconcile.py	2009-09-24 04:54:19 +0000
+++ bzrlib/reconcile.py	2010-01-07 01:30:20 +0000
@@ -27,12 +27,12 @@
 
 
 from bzrlib import (
+    cleanup,
     errors,
     ui,
     repository,
-    repofmt,
     )
-from bzrlib.trace import mutter, note
+from bzrlib.trace import mutter
 from bzrlib.tsort import topo_sort
 from bzrlib.versionedfile import AdapterFactory, FulltextContentFactory
 
@@ -120,15 +120,16 @@
         self.branch = a_branch
 
     def reconcile(self):
+        operation = cleanup.OperationWithCleanups(self._reconcile)
+        self.add_cleanup = operation.add_cleanup
+        operation.run_simple()
+
+    def _reconcile(self):
         self.branch.lock_write()
-        try:
-            self.pb = ui.ui_factory.nested_progress_bar()
-            try:
-                self._reconcile_steps()
-            finally:
-                self.pb.finished()
-        finally:
-            self.branch.unlock()
+        self.add_cleanup(self.branch.unlock)
+        self.pb = ui.ui_factory.nested_progress_bar()
+        self.add_cleanup(self.pb.finished)
+        self._reconcile_steps()
 
     def _reconcile_steps(self):
         self._reconcile_revision_history()
@@ -192,15 +193,16 @@
         garbage_inventories: The number of inventory objects without revisions
                              that were garbage collected.
         """
+        operation = cleanup.OperationWithCleanups(self._reconcile)
+        self.add_cleanup = operation.add_cleanup
+        operation.run_simple()
+
+    def _reconcile(self):
         self.repo.lock_write()
-        try:
-            self.pb = ui.ui_factory.nested_progress_bar()
-            try:
-                self._reconcile_steps()
-            finally:
-                self.pb.finished()
-        finally:
-            self.repo.unlock()
+        self.add_cleanup(self.repo.unlock)
+        self.pb = ui.ui_factory.nested_progress_bar()
+        self.add_cleanup(self.pb.finished)
+        self._reconcile_steps()
 
     def _reconcile_steps(self):
         """Perform the steps to reconcile this repository."""
@@ -502,23 +504,21 @@
         collection = self.repo._pack_collection
         collection.ensure_loaded()
         collection.lock_names()
-        try:
-            packs = collection.all_packs()
-            all_revisions = self.repo.all_revision_ids()
-            total_inventories = len(list(
-                collection.inventory_index.combined_index.iter_all_entries()))
-            if len(all_revisions):
-                new_pack =  self.repo._reconcile_pack(collection, packs,
-                    ".reconcile", all_revisions, self.pb)
-                if new_pack is not None:
-                    self._discard_and_save(packs)
-            else:
-                # only make a new pack when there is data to copy.
+        self.add_cleanup(collection._unlock_names)
+        packs = collection.all_packs()
+        all_revisions = self.repo.all_revision_ids()
+        total_inventories = len(list(
+            collection.inventory_index.combined_index.iter_all_entries()))
+        if len(all_revisions):
+            new_pack =  self.repo._reconcile_pack(collection, packs,
+                ".reconcile", all_revisions, self.pb)
+            if new_pack is not None:
                 self._discard_and_save(packs)
-            self.garbage_inventories = total_inventories - len(list(
-                collection.inventory_index.combined_index.iter_all_entries()))
-        finally:
-            collection._unlock_names()
+        else:
+            # only make a new pack when there is data to copy.
+            self._discard_and_save(packs)
+        self.garbage_inventories = total_inventories - len(list(
+            collection.inventory_index.combined_index.iter_all_entries()))
 
     def _discard_and_save(self, packs):
         """Discard some packs from the repository.

